unit KOLIPHLPAPI;

interface

uses Windows, kol;

const
  MaxAdapterNameLength = 256;
  MaxAdapterDescriptionLength = 128;
  MaxAdapterAddressLength = 8;

type
    // IP Address
  PIPAddressString = ^TIPAddressString;
  TIPAddressString = array [0..15] of char;           // xxx.xxx.xxx.xxx

    // IP Address list
  PIPAddressList = ^TIPAddressList;
  TIPAddressList = record
    Next: PIPAddressList;
    IpAddress: TIPAddressString;
    IpMask: TIPAddressString;
    Context: integer;
  end;

    // Internet Adaptor info
  PIPAdapterInfo = ^TIPAdapterInfo;
  TIPAdapterInfo = record
    Next: PIPAdapterInfo;
    ComboIndex: integer;
    AdapterName: array [0..MaxAdapterNameLength + 3] of char;
    Description: array [0..MaxAdapterDescriptionLength + 3] of char;
    AddressLength: integer;
    Address: array [0..MaxAdapterAddressLength - 1] of byte;
    Index: integer;
    AdapterType: integer;
    DHCPEnabled: integer;
    CurrentIPAddress: PIPAddressList;
    IPAddressList: TIPAddressList;
    GatewayList: TIPAddressList;
    DHCPServer: TIPAddressList;
    HaveWINS: Bool;
    PrimaryWINSServer: TIPAddressList;
    SecondaryWINSServer: TIPAddressList;
    LeaseObtained: integer;
    LeaseExpires: integer;
  end;

    // IP Statistics
  TIPStatistics = record
    IPStats: array [0..22] of integer;
  end;

    // TCP statistics
  TTCPStatistics = record
    TCPStats: array [0..14] of integer;
  end;

    // TCP Table
  TTCPTableEntry = record
    dwState: DWord;
    dwLocalAddr: DWord;
    dwLocalPort: DWord;
    dwRemoteAddr: DWord;
    dwRemotePort: DWord;
  end;

    // UDP Table
  TUDPTableEntry = record
    dwLocalAddr: DWord;
    dwLocalPort: DWord;
  end;

  TUDPTableEntryEx = record
    dwLocalAddr: DWord;
    dwLocalPort: DWord;
    dwProcessID: DWord;
  end;

    // TTCPTableEntryEx
  TTCPTableEntryEx = record
    dwState: DWord;
    dwLocalAddr: DWord;
    dwLocalPort: DWord;
    dwRemoteAddr: DWord;
    dwRemotePort: DWord;
    dwProcessID: DWord;
  end;

  PTCPTable = ^TTCPTable;
  TTCPTable = record
    NumEntries: integer;
    Table: array [0..0] of TTCPTableEntry;
  end;

  PUDPTable = ^TUDPTable;
  TUDPTable = record
    NumEntries: integer;
    Table: array [0..0] of TUDPTableEntry;
  end;

    // TCP Table - extended
  PTCPTableEx = ^TTCPTableEx;
  TTCPTableEx = record
    NumEntries: integer;
    Table: array [0..0] of TTCPTableEntryEx;
  end;

    // UDP Table - extended
  PUDPTableEx = ^TUDPTableEx;
  TUDPTableEx = record
    NumEntries: integer;
    Table: array [0..0] of TUDPTableEntryEx;
  end;

const
  DLLName = 'IPHLPAPI.DLL';

var
  AllocateAndGetTcpExTableFromStack: procedure (var Table: PTCPTableEx;
    bOrder: Bool; Heap: THandle; Zero, Flags: integer); stdcall;
  AllocateAndGetUdpExTableFromStack: procedure (var Table: PUDPTableEx;
    bOrder: Bool; Heap: THandle; Zero, Flags: integer); stdcall;

function GetTcpTableEx: PTCPTableEx;
function GetUdpTableEx: PUDPTableEx;

function GetAdaptersInfo(AdapterInfo: PIPAdapterInfo; var BufLen: integer): integer;
  stdcall; external DLLName;

function GetIpStatistics(var Stats: TIPStatistics): integer; stdcall; external DLLName;

function GetTcpStatistics(var Stats: TTCPStatistics): integer; stdcall; external DLLName;

function GetTcpTable(Table: PTCPTable; var BufLen: integer; Order: Bool): integer;
  stdcall; external DLLName;

function GetUdpTable(Table: PUDPTable; var BufLen: integer; Order: Bool): integer;
  stdcall; external DLLName;

implementation

procedure InitExtendedAPI;
var
  hLib: HModule;
begin
  AllocateAndGetTcpExTableFromStack := nil;
  AllocateAndGetUdpExTableFromStack := nil;

  hLib := LoadLibrary(DLLName);
  if hLib <> 0 then 
  begin
    AllocateAndGetTcpExTableFromStack := GetProcAddress(hLib,
      'AllocateAndGetTcpExTableFromStack');
    AllocateAndGetUdpExTableFromStack := GetProcAddress(hLib,
      'AllocateAndGetUdpExTableFromStack');
  end;
end;

function GetTcpTableEx: PTCPTableEx;
var
  bytes: integer;
  PTable: PTCPTableEx;
begin
  Result := nil;
  if @AllocateAndGetTcpExTableFromStack <> nil then 
  begin
    AllocateAndGetTcpExTableFromStack(PTable, True, GetProcessHeap, 2, 2);
    try
      bytes := HeapSize(GetProcessHeap, 0, PTable);
      GetMem(Result, bytes);
      Move(PTable^, Result^, bytes);
    finally
      HeapFree(GetProcessHeap, 0, PTable);
    end;
  end;
end;

function GetUdpTableEx: PUDPTableEx;
var
  bytes: integer;
  PTable: PUDPTableEx;
begin
  Result := nil;
  if @AllocateAndGetUdpExTableFromStack <> nil then 
  begin
    AllocateAndGetUdpExTableFromStack(PTable, True, GetProcessHeap, 2, 2);
    try
      bytes := HeapSize(GetProcessHeap, 0, PTable);
      GetMem(Result, bytes);
      Move(PTable^, Result^, bytes);
    finally
      HeapFree(GetProcessHeap, 0, PTable);
    end;
  end;
end;

initialization
  InitExtendedAPI;
end.

