unit KolNetworkInfo;

interface

uses
  Windows, Kol, KOLIPHLPAPI;

type
    // Internet Adapter type
  TNetworkAdapterType = (natOther, natEthernet, natTokenRing, natFDDI,
    natPPP, natLoopBack, natSLIP);

  PNetworkAdapter = ^TNetworkAdapter;
  TNetworkAdapter = object(TObj)
  private
    fName: string;
    fDescription: string;
    fMACAddress: string;
    fDriverName: string;
    fIndex: integer;
    fDHCPServerAddress: string;
    fPrimaryWINSServer, fSecondaryWINSServer: string;
    fDHCPEnabled, fHaveWINS: boolean;
    fDefaultGateway: string;
    fAdapterType: TNetworkAdapterType;
    fLeaseObtained, fLeaseExpires: TDateTime;
    fIPAddresses, fIPMasks: PStrList;
  public
    destructor Destroy; virtual;
    property Index: integer read fIndex;
    property Name: string read fName;
    property DriverName: string read fDriverName;
    property Description: string read fDescription;
    property MACAddress: string read fMACAddress;
    property DHCPEnabled: boolean read fDHCPEnabled;
    property DHCPServerAddress: string read fDHCPServerAddress;
    property DHCPStartLease: TDateTime read fLeaseObtained;
    property DHCPEndLease: TDateTime read fLeaseExpires;
    property DefaultGateway: string read fDefaultGateway;
    property HaveWINS: boolean read fHaveWINS;
    property PrimaryWINSServer: string read fPrimaryWINSServer;
    property SecondaryWINSServer: string read fSecondaryWINSServer;
    property AdapterType: TNetworkAdapterType read fAdapterType;
    property IPAddresses: PStrList read fIPAddresses;
    property IPMasks: PStrlist read fIPMasks;
  end;

function NewNetworkAdapter(Info: PIPAdapterInfo): PNetworkAdapter;

type
  PIPStats = ^TIpStats;
  TIPStats = object(Tobj)
  private
    fDummy: integer;
    fIPStats: TIPStatistics;
    function GetIPStats(Index: integer): integer;
  public
    property IPForwarding: integer index 0 read GetIPStats write fDummy;
    property IPDefaultTTL: integer index 1 read GetIPStats write fDummy;
    property IPDatagramsReceived: integer index 2 read GetIPStats write fDummy;
    property IPReceivedHeaderErrors: integer index 3 read GetIPStats write fDummy;
    property IPReceivedAddressErrors: integer index 4 read GetIPStats write fDummy;
    property IPForwardedDatagrams: integer index 5 read GetIPStats write fDummy;
    property IPUnknownProtocols: integer index 6 read GetIPStats write fDummy;
    property IPReceivedDiscardCount: integer index 7 read GetIPStats write fDummy;
    property IPReceivedDeliveredCount: integer index 8 read GetIPStats write fDummy;
    property IPSentDatagramsCount: integer index 9 read GetIPStats write fDummy;
    property IPRoutingDiscards: integer index 10 read GetIPStats write fDummy;
    property IPSetDatagramsDiscarded: integer index 11 read GetIPStats write fDummy;
    property IPNoRouteDatagrams: integer index 12 read GetIPStats write fDummy;
    property IPReassembleTimeoutCount: integer index 13 read GetIPStats write fDummy;
    property IPReassemblyRequiredCount: integer index 14 read GetIPStats write fDummy;
    property IPOKReassemblies: integer index 15 read GetIPStats write fDummy;
    property IPBadReassemblies: integer index 16 read GetIPStats write fDummy;
    property IPOKFragmentations: integer index 17 read GetIPStats write fDummy;
    property IPBadFragmentations: integer index 18 read GetIPStats write fDummy;
    property IPFragmentationsCount: integer index 19 read GetIPStats write fDummy;
    property IPNumInterfaces: integer index 20 read GetIPStats write fDummy;
    property IPNumAddresses: integer index 21 read GetIPStats write fDummy;
    property IPNumRoutes: integer index 22 read GetIPStats write fDummy;
  end;

function NewIpStats: PIPStats;

type
  PTCPStats = ^TTcpStats;
  TTCPStats = object(TObj)
  private
    fDummy: integer;
    fTCPStats: TTCPStatistics;
    function GetTCPStats(Index: integer): integer;
  public
    property TCPTimeoutAlgo: integer index 0 read GetTCPStats write fDummy;
    property TCPMinTimeout: integer index 1 read GetTCPStats write fDummy;
    property TCPMaxTimeout: integer index 2 read GetTCPStats write fDummy;
    property TCPMaxConnections: integer index 3 read GetTCPStats write fDummy;
    property TCPActiveOpens: integer index 4 read GetTCPStats write fDummy;
    property TCPPassiveOpens: integer index 5 read GetTCPStats write fDummy;
    property TCPFailedConnects: integer index 6 read GetTCPStats write fDummy;
    property TCPResetConnects: integer index 7 read GetTCPStats write fDummy;
    property TCPCurrentConnects: integer index 8 read GetTCPStats write fDummy;
    property TCPReceivedSegments: integer index 9 read GetTCPStats write fDummy;
    property TCPSentSegments: integer index 10 read GetTCPStats write fDummy;
    property TCPResentSegments: integer index 11 read GetTCPStats write fDummy;
    property TCPErrorsReceived: integer index 12 read GetTCPStats write fDummy;
    property TCPSentSegsReset: integer index 13 read GetTCPStats write fDummy;
    property TCPCumulativeConnects: integer index 14 read GetTCPStats write fDummy;
  end;
function NewTCPStats: PTCPStats;

type
  PNetworkInfo = ^TNetworkInfo;
  TNetworkInfo = object(TObj)
  private
    { Private declarations }
    fAdapters: PList;
    fIPStatistics, fIPStatsDummy: PIPStats;
    fTCPStatistics, fTCPStatsDummy: PTCPStats;
    fTCPConnectionsDummy, fUDPConnectionsDummy: PStrList;
    fTCPConnections: PStrList;
    fUDPConnections: PStrList;
    fMapProcessIDs: boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    //constructor Create (AOwner: TComponent); override;
    destructor Destroy; virtual;
    procedure InitAdapters;
    procedure ClearAdapters;
    procedure RefreshStatistics;
    procedure RefreshTCPConnections;
    procedure RefreshUDPConnections;
    property Adapters: PList read fAdapters;
    // Statistics
    property IPStatistics: PIPStats read fIPStatistics write fIPStatsDummy;
    property TCPStatistics: PTCPStats read fTCPStatistics write fTCPStatsDummy;
    // Connections
    property TCPConnections: PStrList read fTCPConnections write fTCPConnectionsDummy;
    property UDPConnections: PStrList read fUDPConnections write fUDPConnectionsDummy;
    // Bells and whistles!
    property MapProcessIDs: boolean read fMapProcessIDs write fMapProcessIDs;
  end;
function NewnetworkInfo(Aowner: PControl): PNetworkInfo;

implementation

uses KolProcessMapper;

// Colin Wilson's time_t >> TDateTime converter

function time_tToDateTime(t: integer): TDateTime;
type
  PComp = ^comp;
var
  baseDate: TDateTime;
  systemBaseDate: TSystemTime;
  filetimeBaseDate: TFileTime;
  timet_comp, c: comp;
begin
  {baseDate := }EncodeDate(1970, 1, 1, Basedate);
  DateTime2SystemTime(baseDate, systemBaseDate);
  SystemTimeToFileTime(systemBaseDate, filetimeBaseDate);
  timet_comp := PComp(@filetimeBaseDate)^;
  c := t;
  c := c * 10000000 + timet_comp; // + d3time_comp;
  filetimeBaseDate := PFileTime(@c)^;
  FileTimeToSystemTime(filetimeBaseDate, systemBaseDate);
  {Result := }SystemTime2DateTime(systemBaseDate, Result);
end;

// Colin Wilson's TDateTime >> time_t converter

function DateTimeTotime_t(const dt: TDateTime): DWord;
type
  PComp = ^comp;
var
  baseDate: TDateTime;
  systemBaseDate: TSystemTime;
  filetimeBaseDate: TFileTime;
  timet_comp, c: comp;
begin
  {baseDate := }EncodeDate(1970, 1, 1, BaseDate);
  DateTime2SystemTime(baseDate, systemBaseDate);
  SystemTimeToFileTime(systemBaseDate, filetimeBaseDate);
  timet_comp := PComp(@filetimeBaseDate)^;
  DateTime2SystemTime(dt, systemBaseDate);
  SystemTimeToFileTime(systemBaseDate, filetimeBaseDate);
  c := PComp(@filetimeBaseDate)^;
  c := (c - timet_comp) / 10000000;
  Result := Round(c);
end;

// Transmogrify port number from network byte order to host byte order

function PortNumToString(PortNum: word): string;
begin
  Result := Int2Str((LoByte(PortNum) * 256) + HiByte(PortNum));
end;

// Convert a packed IP address to xxx.xxx.xxx.xxx format

function IPAddressToString(IPAddress: DWord): string;
var
  Idx: integer;
begin
  Result := '';
  for Idx := 0 to 3 do 
  begin
    Result := Result + Format('%d', [IPAddress and $FF]);
    if Idx <> 3 then Result := Result + '.';
    IPAddress := IPAddress shr 8;
  end;
end;

// Create a human-readable MAC address

function MACAddressFromBytes(Count: integer; Bytes: PChar): string;
begin
  Result := '';
  while Count > 0 do 
  begin
    Result := Result + Int2Hex(Ord(Bytes^), 2);
    if Count > 1 then Result := Result + '-';
    Inc(Bytes);  
    Dec(Count);
  end;
end;

// TNetworkAdapter

function NewNetworkAdapter(Info: PIPAdapterInfo): PNetworkAdapter;
var
  HyphenPos: integer;
  IPAddr: PIPAddressList;
begin
  New(Result, Create);
  with Result^ do
  begin
    fIPMasks := Newstrlist;
    fIPAddresses := Newstrlist;
    fName := Info.AdapterName;

    HyphenPos := Pos('-', Info.Description);
    if HyphenPos = 0 then 
    begin
      fDescription := Info.Description;
      fDriverName := '';
    end 
    else 
    begin
      fDescription := Trim(Copy(Info.Description, 1, HyphenPos - 1));
      fDriverName := Trim(Copy(Info.Description, HyphenPos + 1, MaxInt));
    end;

    fMACAddress := MACAddressFromBytes(Info.AddressLength, @Info.Address);
    fIndex := Info.Index;
    fAdapterType := TNetworkAdapterType(Info.AdapterType);
    fDHCPEnabled := Info.DHCPEnabled <> 0;
    fHaveWINS := Info.HaveWINS <> False;
    fDefaultGateway := Info.GatewayList.IpAddress;

    IPAddr := @Info.IPAddressList;
    while IPAddr <> nil do 
    begin
      fIPAddresses.Add(IPAddr.IpAddress);
      fIPMasks.Add(IPAddr.IpMask);
      IPAddr := IPAddr.Next;
    end;

    if fDHCPEnabled then 
    begin
      fDHCPServerAddress := Info.DHCPServer.IpAddress;
      fLeaseObtained := time_tToDateTime(Info.LeaseObtained);
      fLeaseExpires := time_tToDateTime(Info.LeaseExpires);
    end;

    if fHaveWINS then 
    begin
      fPrimaryWINSServer := Info.PrimaryWINSServer.IpAddress;
      fSecondaryWINSServer := Info.SecondaryWINSServer.IPAddress;
    end;
  end;
end;

destructor TNetworkAdapter.Destroy;
begin
  fIPMasks.Free;
  fIPAddresses.Free;
  inherited Destroy;
end;

// TIPStats

function TIPStats.GetIPStats(Index: integer): integer;
begin
  Result := fIPStats.IPStats[Index];
end;

// TTCPStats

function TTCPStats.GetTCPStats(Index: integer): integer;
begin
  Result := fTCPStats.TCPStats[Index];
end;

// TNetworkInfo

function NewNetworkInfo(AOwner: PControl): PNetworkInfo;
begin
  New(Result, Create);
  Aowner.add2autofree(Result);
  with Result^ do
  begin
    fAdapters := NewList;
    fTCPConnections := Newstrlist;
    fUDPConnections := Newstrlist;
    InitAdapters;
    RefreshStatistics;
    RefreshTCPConnections;
    RefreshUDPConnections;
  end;
end;

destructor TNetworkInfo.Destroy;
begin
  ClearAdapters;
  fAdapters.Free;
  fIPStatistics.Free;
  fTCPStatistics.Free;
  fTCPConnections.Free;
  fUDPConnections.Free;
  inherited Destroy;
end;

procedure TNetworkInfo.ClearAdapters;
begin
  while fAdapters.Count > 0 do 
  begin
    PNetworkAdapter(fAdapters.Items[0]).Free;
    fAdapters.Delete(0);
  end;
end;

procedure TNetworkInfo.InitAdapters;
var
  BuffLen: integer;
  Buff, Curr: PIPAdapterInfo;
begin
  // Clear current adapter list
  ClearAdapters;

  // Figure out size of the wanted buffer
  BuffLen := 0;
  GetAdaptersInfo(nil, BuffLen);
  if Bufflen = 0 then exit;
  // Allocate the buffer and load it
  Buff := AllocMem(BuffLen);
  try
    GetAdaptersInfo(Buff, BuffLen);
    Curr := Buff;

    // Iterate through the linked list of adapters
    while Curr <> nil do
    begin
      fAdapters.Add(NewNetworkAdapter(Curr));
      Curr := Curr.Next;
    end;
  finally
    FreeMem(Buff);
  end;
end;

procedure TNetworkInfo.RefreshTCPConnections;
var
  TableEx: PTCPTableEx;

  procedure LoadTCPTable;
  var
    Table: PTCPTable;
    Idx, BuffLen: integer;
  begin
    BuffLen := 0;
    GetTCPTable(nil, BuffLen, False);
    if bufflen = 0 then exit;
    Table := AllocMem(BuffLen);
    try
      GetTCPTable(Table, BuffLen, True);
      for Idx := 0 to Table.NumEntries - 1 do 
      begin
        fTCPConnections.Add(Int2Str(Table.Table[Idx].dwState) + '|' +
          IPAddressToString(Table.Table[Idx].dwLocalAddr) + '|' +
          PortNumToString(Table.Table[Idx].dwLocalPort) +
          '|' +
          IPAddressToString(Table.Table[Idx].dwRemoteAddr) +
          '|' +
          PortNumToString(Table.Table[Idx].dwRemotePort) + '|-');
      end;
    finally
      FreeMem(Table);
    end;
  end;

  procedure LoadTCPTableEx;
  var
    Idx: integer;
  begin
    if fMapProcessIDs then ProcessCreateSnapshot;
    for Idx := 0 to TableEx.NumEntries - 1 do
      fTCPConnections.Add(Int2Str(TableEx.Table[Idx].dwState) + '|' +
        IPAddressToString(TableEx.Table[Idx].dwLocalAddr) +
        '|' +
        PortNumToString(TableEx.Table[Idx].dwLocalPort) + '|' +
        IPAddressToString(TableEx.Table[Idx].dwRemoteAddr) + '|' +
        PortNumToString(TableEx.Table[Idx].dwRemotePort) + '|' +
        ProcessPIDToProcessName(TableEx.Table[Idx].dwProcessID));
                                 
    if fMapProcessIDs then ProcessDeleteSnapshot;
  end;
begin
  fTCPConnections.Clear;
  TableEx := GetTcpTableEx;
  if TableEx = nil then LoadTCPTable 
  else 
    try
      LoadTCPTableEx;
    finally
      FreeMem(TableEx);
    end;
end;

procedure TNetworkInfo.RefreshUDPConnections;
var
  TableEx: PUDPTableEx;

  procedure LoadUDPTable;
  var
    Table: PUDPTable;
    Idx, BuffLen: integer;
  begin
    BuffLen := 0;
    GetUDPTable(nil, BuffLen, False);
    if bufflen = 0 then exit;
    Table := AllocMem(BuffLen);
    try
      GetUDPTable(Table, BuffLen, True);
      for Idx := 0 to Table.NumEntries - 1 do
        fUDPConnections.Add(IPAddressToString(Table.Table[Idx].dwLocalAddr) + '|' +
          PortNumToString(Table.Table[Idx].dwLocalPort) + '|-');

    finally
      FreeMem(Table);
    end;
  end;

  procedure LoadUDPTableEx;
  var
    Idx: integer;
  begin
    if fMapProcessIDs then ProcessCreateSnapshot;
    for Idx := 0 to TableEx.NumEntries - 1 do
      fUDPConnections.Add(IPAddressToString(TableEx.Table[Idx].dwLocalAddr) + '|' +
        PortNumToString(TableEx.Table[Idx].dwLocalPort) + '|' +
        ProcessPIDToProcessName(TableEx.Table[Idx].dwProcessID));
                                 
    if fMapProcessIDs then ProcessDeleteSnapshot;
  end;
begin
  fUDPConnections.Clear;
  TableEx := GetUdpTableEx;
  if TableEx = nil then LoadUDPTable 
  else 
    try
      LoadUDPTableEx;
    finally
      FreeMem(TableEx);
    end;
end;

procedure TNetworkInfo.RefreshStatistics;
begin
  if fIPStatistics = nil then fIPStatistics := NewIPStats;
  GetIpStatistics(fIPStatistics.fIPStats);
  if fTCPStatistics = nil then fTCPStatistics := NewTCPStats;
  GetTCPStatistics(fTCPStatistics.fTCPStats);
end;

function NewIPStats: PIPstats;
begin
  New(Result, Create);
end;

function NewTCPStats: PTCPstats;
begin
  New(Result, Create);
end;

end.

